import yaml, json
import argparse
import pathlib

yaml.add_representer(dict, lambda self, data: yaml.representer.SafeRepresenter.represent_dict(self, data.items())) # thanks to https://stackoverflow.com/a/52268128

class convert:
    
    
    def __init__(self, directory = "", output = "yaml", auto = True):
        if directory != "":
            self.directory =pathlib.Path(directory)
        else:
            self.directory = pathlib.Path.cwd()
        self.output = output
        if self.output != "yaml" and self.output != "json":
            print("Error! output type not supported.")
            return False
        if auto:
            self.iterate_files(self.directory)

        
    def iterate_files(self,d):
        if self.output == "yaml":
            suffixes = [".json"]
        elif self.output == "json":
            suffixes = [".yml", ".yaml"]
        else:
            return
        
        for p in d.iterdir():
            if p.is_dir() and not p.is_symlink():
                self.iterate_files(p)
            elif p.is_file() and p.suffix in suffixes:
                self.convert_file(p)
        
    def convert_file(self,p):
        new_file = p.with_suffix("." + self.output)
        if p.stat().st_size == 0:
            new_file.write_text("")
            return
        with p.open() as f:
            contents = p.read_text()
        try: 
            if self.output == "yaml":
                obj = json.loads(contents)
                output_text = yaml.dump(obj)
            else:
                obj = yaml.load(contents,Loader=yaml.FullLoader)
                output_text = json.dumps(obj,indent=2)
            new_file.write_text(output_text)
                
        except Exception as e:
            print("Error converting: {}".format(str(p)))
            print(e)
            
if __name__ == "__main__": 
    parser = argparse.ArgumentParser(description="JSON vs. YAML coverter. Will convert JSON to YAML or YAML to JSON")

    parser.add_argument("--path", "-p", default="", dest='path', help='path to the directory or file to convert')
    parser.add_argument("--output", "-o", default="yaml", dest='output', choices=["json","yaml"], help='what to convert to; json implies coverting yaml->json, yaml implies converting json->yaml')

    args = parser.parse_args()
    convert(directory = args.path, output=args.output)
